@@ -2,7 +2,7 @@
 'use strict';
 
 // SDP helpers.
-var SDPUtils = {};
+const SDPUtils = {};
 
 // Generate an alphanumeric identifier for cname or mids.
 // TODO: use UUIDs instead? https://gist.github.com/jed/982883
@@ -15,43 +15,39 @@ SDPUtils.localCName = SDPUtils.generateIdentifier();
 
 // Splits SDP into lines, dealing with both CRLF and LF.
 SDPUtils.splitLines = function(blob) {
-  return blob.trim().split('\n').map(function(line) {
-    return line.trim();
-  });
+  return blob.trim().split('\n').map(line => line.trim());
 };
 // Splits SDP into sessionpart and mediasections. Ensures CRLF.
 SDPUtils.splitSections = function(blob) {
-  var parts = blob.split('\nm=');
-  return parts.map(function(part, index) {
-    return (index > 0 ? 'm=' + part : part).trim() + '\r\n';
-  });
+  const parts = blob.split('\nm=');
+  return parts.map((part, index) => (index > 0 ?
+    'm=' + part : part).trim() + '\r\n');
 };
 
-// returns the session description.
+// Returns the session description.
 SDPUtils.getDescription = function(blob) {
-  var sections = SDPUtils.splitSections(blob);
+  const sections = SDPUtils.splitSections(blob);
   return sections && sections[0];
 };
 
-// returns the individual media sections.
+// Returns the individual media sections.
 SDPUtils.getMediaSections = function(blob) {
-  var sections = SDPUtils.splitSections(blob);
+  const sections = SDPUtils.splitSections(blob);
   sections.shift();
   return sections;
 };
 
 // Returns lines that start with a certain prefix.
 SDPUtils.matchPrefix = function(blob, prefix) {
-  return SDPUtils.splitLines(blob).filter(function(line) {
-    return line.indexOf(prefix) === 0;
-  });
+  return SDPUtils.splitLines(blob).filter(line => line.indexOf(prefix) === 0);
 };
 
 // Parses an ICE candidate line. Sample input:
 // candidate:702786350 2 udp 41819902 8.8.8.8 60769 typ relay raddr 8.8.8.8
 // rport 55996"
+// Input can be prefixed with a=.
 SDPUtils.parseCandidate = function(line) {
-  var parts;
+  let parts;
   // Parse both variants.
   if (line.indexOf('a=candidate:') === 0) {
     parts = line.substring(12).split(' ');
@@ -59,19 +55,19 @@ SDPUtils.parseCandidate = function(line) {
     parts = line.substring(10).split(' ');
   }
 
-  var candidate = {
+  const candidate = {
     foundation: parts[0],
-    component: parseInt(parts[1], 10),
+    component: {1: 'rtp', 2: 'rtcp'}[parts[1]] || parts[1],
     protocol: parts[2].toLowerCase(),
     priority: parseInt(parts[3], 10),
     ip: parts[4],
     address: parts[4], // address is an alias for ip.
     port: parseInt(parts[5], 10),
     // skip parts[6] == 'typ'
-    type: parts[7]
+    type: parts[7],
   };
 
-  for (var i = 8; i < parts.length; i += 2) {
+  for (let i = 8; i < parts.length; i += 2) {
     switch (parts[i]) {
       case 'raddr':
         candidate.relatedAddress = parts[i + 1];
@@ -83,11 +79,13 @@ SDPUtils.parseCandidate = function(line) {
         candidate.tcpType = parts[i + 1];
         break;
       case 'ufrag':
-        candidate.ufrag = parts[i + 1]; // for backward compability.
+        candidate.ufrag = parts[i + 1]; // for backward compatibility.
         candidate.usernameFragment = parts[i + 1];
         break;
-      default: // extension handling, in particular ufrag
-        candidate[parts[i]] = parts[i + 1];
+      default: // extension handling, in particular ufrag. Don't overwrite.
+        if (candidate[parts[i]] === undefined) {
+          candidate[parts[i]] = parts[i + 1];
+        }
         break;
     }
   }
@@ -95,16 +93,25 @@ SDPUtils.parseCandidate = function(line) {
 };
 
 // Translates a candidate object into SDP candidate attribute.
+// This does not include the a= prefix!
 SDPUtils.writeCandidate = function(candidate) {
-  var sdp = [];
+  const sdp = [];
   sdp.push(candidate.foundation);
-  sdp.push(candidate.component);
+
+  const component = candidate.component;
+  if (component === 'rtp') {
+    sdp.push(1);
+  } else if (component === 'rtcp') {
+    sdp.push(2);
+  } else {
+    sdp.push(component);
+  }
   sdp.push(candidate.protocol.toUpperCase());
   sdp.push(candidate.priority);
   sdp.push(candidate.address || candidate.ip);
   sdp.push(candidate.port);
 
-  var type = candidate.type;
+  const type = candidate.type;
   sdp.push('typ');
   sdp.push(type);
   if (type !== 'host' && candidate.relatedAddress &&
@@ -126,17 +133,18 @@ SDPUtils.writeCandidate = function(candidate) {
 };
 
 // Parses an ice-options line, returns an array of option tags.
+// Sample input:
 // a=ice-options:foo bar
 SDPUtils.parseIceOptions = function(line) {
   return line.substr(14).split(' ');
 };
 
-// Parses an rtpmap line, returns RTCRtpCoddecParameters. Sample input:
+// Parses a rtpmap line, returns RTCRtpCoddecParameters. Sample input:
 // a=rtpmap:111 opus/48000/2
 SDPUtils.parseRtpMap = function(line) {
-  var parts = line.substr(9).split(' ');
-  var parsed = {
-    payloadType: parseInt(parts.shift(), 10) // was: id
+  let parts = line.substr(9).split(' ');
+  const parsed = {
+    payloadType: parseInt(parts.shift(), 10), // was: id
   };
 
   parts = parts[0].split('/');
@@ -149,31 +157,31 @@ SDPUtils.parseRtpMap = function(line) {
   return parsed;
 };
 
-// Generate an a=rtpmap line from RTCRtpCodecCapability or
+// Generates a rtpmap line from RTCRtpCodecCapability or
 // RTCRtpCodecParameters.
 SDPUtils.writeRtpMap = function(codec) {
-  var pt = codec.payloadType;
+  let pt = codec.payloadType;
   if (codec.preferredPayloadType !== undefined) {
     pt = codec.preferredPayloadType;
   }
-  var channels = codec.channels || codec.numChannels || 1;
+  const channels = codec.channels || codec.numChannels || 1;
   return 'a=rtpmap:' + pt + ' ' + codec.name + '/' + codec.clockRate +
       (channels !== 1 ? '/' + channels : '') + '\r\n';
 };
 
-// Parses an a=extmap line (headerextension from RFC 5285). Sample input:
+// Parses a extmap line (headerextension from RFC 5285). Sample input:
 // a=extmap:2 urn:ietf:params:rtp-hdrext:toffset
 // a=extmap:2/sendonly urn:ietf:params:rtp-hdrext:toffset
 SDPUtils.parseExtmap = function(line) {
-  var parts = line.substr(9).split(' ');
+  const parts = line.substr(9).split(' ');
   return {
     id: parseInt(parts[0], 10),
     direction: parts[0].indexOf('/') > 0 ? parts[0].split('/')[1] : 'sendrecv',
-    uri: parts[1]
+    uri: parts[1],
   };
 };
 
-// Generates a=extmap line from RTCRtpHeaderExtensionParameters or
+// Generates an extmap line from RTCRtpHeaderExtensionParameters or
 // RTCRtpHeaderExtension.
 SDPUtils.writeExtmap = function(headerExtension) {
   return 'a=extmap:' + (headerExtension.id || headerExtension.preferredId) +
@@ -183,31 +191,31 @@ SDPUtils.writeExtmap = function(headerExtension) {
       ' ' + headerExtension.uri + '\r\n';
 };
 
-// Parses an ftmp line, returns dictionary. Sample input:
+// Parses a fmtp line, returns dictionary. Sample input:
 // a=fmtp:96 vbr=on;cng=on
 // Also deals with vbr=on; cng=on
 SDPUtils.parseFmtp = function(line) {
-  var parsed = {};
-  var kv;
-  var parts = line.substr(line.indexOf(' ') + 1).split(';');
-  for (var j = 0; j < parts.length; j++) {
+  const parsed = {};
+  let kv;
+  const parts = line.substr(line.indexOf(' ') + 1).split(';');
+  for (let j = 0; j < parts.length; j++) {
     kv = parts[j].trim().split('=');
     parsed[kv[0].trim()] = kv[1];
   }
   return parsed;
 };
 
-// Generates an a=ftmp line from RTCRtpCodecCapability or RTCRtpCodecParameters.
+// Generates a fmtp line from RTCRtpCodecCapability or RTCRtpCodecParameters.
 SDPUtils.writeFmtp = function(codec) {
-  var line = '';
-  var pt = codec.payloadType;
+  let line = '';
+  let pt = codec.payloadType;
   if (codec.preferredPayloadType !== undefined) {
     pt = codec.preferredPayloadType;
   }
   if (codec.parameters && Object.keys(codec.parameters).length) {
-    var params = [];
-    Object.keys(codec.parameters).forEach(function(param) {
-      if (codec.parameters[param]) {
+    const params = [];
+    Object.keys(codec.parameters).forEach(param => {
+      if (codec.parameters[param] !== undefined) {
         params.push(param + '=' + codec.parameters[param]);
       } else {
         params.push(param);
@@ -218,25 +226,26 @@ SDPUtils.writeFmtp = function(codec) {
   return line;
 };
 
-// Parses an rtcp-fb line, returns RTCPRtcpFeedback object. Sample input:
+// Parses a rtcp-fb line, returns RTCPRtcpFeedback object. Sample input:
 // a=rtcp-fb:98 nack rpsi
 SDPUtils.parseRtcpFb = function(line) {
-  var parts = line.substr(line.indexOf(' ') + 1).split(' ');
+  const parts = line.substr(line.indexOf(' ') + 1).split(' ');
   return {
     type: parts.shift(),
-    parameter: parts.join(' ')
+    parameter: parts.join(' '),
   };
 };
+
 // Generate a=rtcp-fb lines from RTCRtpCodecCapability or RTCRtpCodecParameters.
 SDPUtils.writeRtcpFb = function(codec) {
-  var lines = '';
-  var pt = codec.payloadType;
+  let lines = '';
+  let pt = codec.payloadType;
   if (codec.preferredPayloadType !== undefined) {
     pt = codec.preferredPayloadType;
   }
   if (codec.rtcpFeedback && codec.rtcpFeedback.length) {
     // FIXME: special handling for trr-int?
-    codec.rtcpFeedback.forEach(function(fb) {
+    codec.rtcpFeedback.forEach(fb => {
       lines += 'a=rtcp-fb:' + pt + ' ' + fb.type +
       (fb.parameter && fb.parameter.length ? ' ' + fb.parameter : '') +
           '\r\n';
@@ -245,14 +254,14 @@ SDPUtils.writeRtcpFb = function(codec) {
   return lines;
 };
 
-// Parses an RFC 5576 ssrc media attribute. Sample input:
+// Parses a RFC 5576 ssrc media attribute. Sample input:
 // a=ssrc:3735928559 cname:something
 SDPUtils.parseSsrcMedia = function(line) {
-  var sp = line.indexOf(' ');
-  var parts = {
-    ssrc: parseInt(line.substr(7, sp - 7), 10)
+  const sp = line.indexOf(' ');
+  const parts = {
+    ssrc: parseInt(line.substr(7, sp - 7), 10),
   };
-  var colon = line.indexOf(':', sp);
+  const colon = line.indexOf(':', sp);
   if (colon > -1) {
     parts.attribute = line.substr(sp + 1, colon - sp - 1);
     parts.value = line.substr(colon + 1);
@@ -262,30 +271,31 @@ SDPUtils.parseSsrcMedia = function(line) {
   return parts;
 };
 
+// Parse a ssrc-group line (see RFC 5576). Sample input:
+// a=ssrc-group:semantics 12 34
 SDPUtils.parseSsrcGroup = function(line) {
-  var parts = line.substr(13).split(' ');
+  const parts = line.substr(13).split(' ');
   return {
     semantics: parts.shift(),
-    ssrcs: parts.map(function(ssrc) {
-      return parseInt(ssrc, 10);
-    })
+    ssrcs: parts.map(ssrc => parseInt(ssrc, 10)),
   };
 };
 
 // Extracts the MID (RFC 5888) from a media section.
-// returns the MID or undefined if no mid line was found.
+// Returns the MID or undefined if no mid line was found.
 SDPUtils.getMid = function(mediaSection) {
-  var mid = SDPUtils.matchPrefix(mediaSection, 'a=mid:')[0];
+  const mid = SDPUtils.matchPrefix(mediaSection, 'a=mid:')[0];
   if (mid) {
     return mid.substr(6);
   }
 };
 
+// Parses a fingerprint line for DTLS-SRTP.
 SDPUtils.parseFingerprint = function(line) {
-  var parts = line.substr(14).split(' ');
+  const parts = line.substr(14).split(' ');
   return {
     algorithm: parts[0].toLowerCase(), // algorithm is case-sensitive in Edge.
-    value: parts[1]
+    value: parts[1].toUpperCase(), // the definition is upper-case in RFC 4572.
   };
 };
 
@@ -293,20 +303,19 @@ SDPUtils.parseFingerprint = function(line) {
 // FIXME: for consistency with other functions this should only
 //   get the fingerprint line as input. See also getIceParameters.
 SDPUtils.getDtlsParameters = function(mediaSection, sessionpart) {
-  var lines = SDPUtils.matchPrefix(mediaSection + sessionpart,
+  const lines = SDPUtils.matchPrefix(mediaSection + sessionpart,
     'a=fingerprint:');
-  // Note: a=setup line is ignored since we use the 'auto' role.
-  // Note2: 'algorithm' is not case sensitive except in Edge.
+  // Note: a=setup line is ignored since we use the 'auto' role in Edge.
   return {
     role: 'auto',
-    fingerprints: lines.map(SDPUtils.parseFingerprint)
+    fingerprints: lines.map(SDPUtils.parseFingerprint),
   };
 };
 
 // Serializes DTLS parameters to SDP.
 SDPUtils.writeDtlsParameters = function(params, setupType) {
-  var sdp = 'a=setup:' + setupType + '\r\n';
-  params.fingerprints.forEach(function(fp) {
+  let sdp = 'a=setup:' + setupType + '\r\n';
+  params.fingerprints.forEach(fp => {
     sdp += 'a=fingerprint:' + fp.algorithm + ' ' + fp.value + '\r\n';
   });
   return sdp;
@@ -315,7 +324,7 @@ SDPUtils.writeDtlsParameters = function(params, setupType) {
 // Parses a=crypto lines into
 //   https://rawgit.com/aboba/edgertc/master/msortc-rs4.html#dictionary-rtcsrtpsdesparameters-members
 SDPUtils.parseCryptoLine = function(line) {
-  var parts = line.substr(9).split(' ');
+  const parts = line.substr(9).split(' ');
   return {
     tag: parseInt(parts[0], 10),
     cryptoSuite: parts[1],
@@ -340,7 +349,7 @@ SDPUtils.parseCryptoKeyParams = function(keyParams) {
   if (keyParams.indexOf('inline:') !== 0) {
     return null;
   }
-  var parts = keyParams.substr(7).split('|');
+  const parts = keyParams.substr(7).split('|');
   return {
     keyMethod: 'inline',
     keySalt: parts[0],
@@ -359,9 +368,9 @@ SDPUtils.writeCryptoKeyParams = function(keyParams) {
       : '');
 };
 
-// Extracts all SDES paramters.
+// Extracts all SDES parameters.
 SDPUtils.getCryptoParameters = function(mediaSection, sessionpart) {
-  var lines = SDPUtils.matchPrefix(mediaSection + sessionpart,
+  const lines = SDPUtils.matchPrefix(mediaSection + sessionpart,
     'a=crypto:');
   return lines.map(SDPUtils.parseCryptoLine);
 };
@@ -370,9 +379,9 @@ SDPUtils.getCryptoParameters = function(mediaSection, sessionpart) {
 // FIXME: for consistency with other functions this should only
 //   get the ice-ufrag and ice-pwd lines as input.
 SDPUtils.getIceParameters = function(mediaSection, sessionpart) {
-  var ufrag = SDPUtils.matchPrefix(mediaSection + sessionpart,
+  const ufrag = SDPUtils.matchPrefix(mediaSection + sessionpart,
     'a=ice-ufrag:')[0];
-  var pwd = SDPUtils.matchPrefix(mediaSection + sessionpart,
+  const pwd = SDPUtils.matchPrefix(mediaSection + sessionpart,
     'a=ice-pwd:')[0];
   if (!(ufrag && pwd)) {
     return null;
@@ -385,27 +394,31 @@ SDPUtils.getIceParameters = function(mediaSection, sessionpart) {
 
 // Serializes ICE parameters to SDP.
 SDPUtils.writeIceParameters = function(params) {
-  return 'a=ice-ufrag:' + params.usernameFragment + '\r\n' +
+  let sdp = 'a=ice-ufrag:' + params.usernameFragment + '\r\n' +
       'a=ice-pwd:' + params.password + '\r\n';
+  if (params.iceLite) {
+    sdp += 'a=ice-lite\r\n';
+  }
+  return sdp;
 };
 
 // Parses the SDP media section and returns RTCRtpParameters.
 SDPUtils.parseRtpParameters = function(mediaSection) {
-  var description = {
+  const description = {
     codecs: [],
     headerExtensions: [],
     fecMechanisms: [],
-    rtcp: []
+    rtcp: [],
   };
-  var lines = SDPUtils.splitLines(mediaSection);
-  var mline = lines[0].split(' ');
-  for (var i = 3; i < mline.length; i++) { // find all codecs from mline[3..]
-    var pt = mline[i];
-    var rtpmapline = SDPUtils.matchPrefix(
+  const lines = SDPUtils.splitLines(mediaSection);
+  const mline = lines[0].split(' ');
+  for (let i = 3; i < mline.length; i++) { // find all codecs from mline[3..]
+    const pt = mline[i];
+    const rtpmapline = SDPUtils.matchPrefix(
       mediaSection, 'a=rtpmap:' + pt + ' ')[0];
     if (rtpmapline) {
-      var codec = SDPUtils.parseRtpMap(rtpmapline);
-      var fmtps = SDPUtils.matchPrefix(
+      const codec = SDPUtils.parseRtpMap(rtpmapline);
+      const fmtps = SDPUtils.matchPrefix(
         mediaSection, 'a=fmtp:' + pt + ' ');
       // Only the first a=fmtp:<pt> is considered.
       codec.parameters = fmtps.length ? SDPUtils.parseFmtp(fmtps[0]) : {};
@@ -424,7 +437,7 @@ SDPUtils.parseRtpParameters = function(mediaSection) {
       }
     }
   }
-  SDPUtils.matchPrefix(mediaSection, 'a=extmap:').forEach(function(line) {
+  SDPUtils.matchPrefix(mediaSection, 'a=extmap:').forEach(line => {
     description.headerExtensions.push(SDPUtils.parseExtmap(line));
   });
   // FIXME: parse rtcp.
@@ -434,13 +447,13 @@ SDPUtils.parseRtpParameters = function(mediaSection) {
 // Generates parts of the SDP media section describing the capabilities /
 // parameters.
 SDPUtils.writeRtpDescription = function(kind, caps) {
-  var sdp = '';
+  let sdp = '';
 
   // Build the mline.
   sdp += 'm=' + kind + ' ';
   sdp += caps.codecs.length > 0 ? '9' : '0'; // reject if no codecs.
   sdp += ' UDP/TLS/RTP/SAVPF ';
-  sdp += caps.codecs.map(function(codec) {
+  sdp += caps.codecs.map(codec => {
     if (codec.preferredPayloadType !== undefined) {
       return codec.preferredPayloadType;
     }
@@ -451,13 +464,13 @@ SDPUtils.writeRtpDescription = function(kind, caps) {
   sdp += 'a=rtcp:9 IN IP4 0.0.0.0\r\n';
 
   // Add a=rtpmap lines for each codec. Also fmtp and rtcp-fb.
-  caps.codecs.forEach(function(codec) {
+  caps.codecs.forEach(codec => {
     sdp += SDPUtils.writeRtpMap(codec);
     sdp += SDPUtils.writeFmtp(codec);
     sdp += SDPUtils.writeRtcpFb(codec);
   });
-  var maxptime = 0;
-  caps.codecs.forEach(function(codec) {
+  let maxptime = 0;
+  caps.codecs.forEach(codec => {
     if (codec.maxptime > maxptime) {
       maxptime = codec.maxptime;
     }
@@ -465,10 +478,9 @@ SDPUtils.writeRtpDescription = function(kind, caps) {
   if (maxptime > 0) {
     sdp += 'a=maxptime:' + maxptime + '\r\n';
   }
-  sdp += 'a=rtcp-mux\r\n';
 
   if (caps.headerExtensions) {
-    caps.headerExtensions.forEach(function(extension) {
+    caps.headerExtensions.forEach(extension => {
       sdp += SDPUtils.writeExtmap(extension);
     });
   }
@@ -479,38 +491,32 @@ SDPUtils.writeRtpDescription = function(kind, caps) {
 // Parses the SDP media section and returns an array of
 // RTCRtpEncodingParameters.
 SDPUtils.parseRtpEncodingParameters = function(mediaSection) {
-  var encodingParameters = [];
-  var description = SDPUtils.parseRtpParameters(mediaSection);
-  var hasRed = description.fecMechanisms.indexOf('RED') !== -1;
-  var hasUlpfec = description.fecMechanisms.indexOf('ULPFEC') !== -1;
+  const encodingParameters = [];
+  const description = SDPUtils.parseRtpParameters(mediaSection);
+  const hasRed = description.fecMechanisms.indexOf('RED') !== -1;
+  const hasUlpfec = description.fecMechanisms.indexOf('ULPFEC') !== -1;
 
   // filter a=ssrc:... cname:, ignore PlanB-msid
-  var ssrcs = SDPUtils.matchPrefix(mediaSection, 'a=ssrc:')
-    .map(function(line) {
-      return SDPUtils.parseSsrcMedia(line);
-    })
-    .filter(function(parts) {
-      return parts.attribute === 'cname';
-    });
-  var primarySsrc = ssrcs.length > 0 && ssrcs[0].ssrc;
-  var secondarySsrc;
+  const ssrcs = SDPUtils.matchPrefix(mediaSection, 'a=ssrc:')
+    .map(line => SDPUtils.parseSsrcMedia(line))
+    .filter(parts => parts.attribute === 'cname');
+  const primarySsrc = ssrcs.length > 0 && ssrcs[0].ssrc;
+  let secondarySsrc;
 
-  var flows = SDPUtils.matchPrefix(mediaSection, 'a=ssrc-group:FID')
-    .map(function(line) {
-      var parts = line.substr(17).split(' ');
-      return parts.map(function(part) {
-        return parseInt(part, 10);
-      });
+  const flows = SDPUtils.matchPrefix(mediaSection, 'a=ssrc-group:FID')
+    .map(line => {
+      const parts = line.substr(17).split(' ');
+      return parts.map(part => parseInt(part, 10));
     });
   if (flows.length > 0 && flows[0].length > 1 && flows[0][0] === primarySsrc) {
     secondarySsrc = flows[0][1];
   }
 
-  description.codecs.forEach(function(codec) {
+  description.codecs.forEach(codec => {
     if (codec.name.toUpperCase() === 'RTX' && codec.parameters.apt) {
-      var encParam = {
+      let encParam = {
         ssrc: primarySsrc,
-        codecPayloadType: parseInt(codec.parameters.apt, 10)
+        codecPayloadType: parseInt(codec.parameters.apt, 10),
       };
       if (primarySsrc && secondarySsrc) {
         encParam.rtx = {ssrc: secondarySsrc};
@@ -520,7 +526,7 @@ SDPUtils.parseRtpEncodingParameters = function(mediaSection) {
         encParam = JSON.parse(JSON.stringify(encParam));
         encParam.fec = {
           ssrc: primarySsrc,
-          mechanism: hasUlpfec ? 'red+ulpfec' : 'red'
+          mechanism: hasUlpfec ? 'red+ulpfec' : 'red',
         };
         encodingParameters.push(encParam);
       }
@@ -528,12 +534,12 @@ SDPUtils.parseRtpEncodingParameters = function(mediaSection) {
   });
   if (encodingParameters.length === 0 && primarySsrc) {
     encodingParameters.push({
-      ssrc: primarySsrc
+      ssrc: primarySsrc,
     });
   }
 
   // we support both b=AS and b=TIAS but interpret AS as TIAS.
-  var bandwidth = SDPUtils.matchPrefix(mediaSection, 'b=');
+  let bandwidth = SDPUtils.matchPrefix(mediaSection, 'b=');
   if (bandwidth.length) {
     if (bandwidth[0].indexOf('b=TIAS:') === 0) {
       bandwidth = parseInt(bandwidth[0].substr(7), 10);
@@ -544,7 +550,7 @@ SDPUtils.parseRtpEncodingParameters = function(mediaSection) {
     } else {
       bandwidth = undefined;
     }
-    encodingParameters.forEach(function(params) {
+    encodingParameters.forEach(params => {
       params.maxBitrate = bandwidth;
     });
   }
@@ -553,17 +559,13 @@ SDPUtils.parseRtpEncodingParameters = function(mediaSection) {
 
 // parses http://draft.ortc.org/#rtcrtcpparameters*
 SDPUtils.parseRtcpParameters = function(mediaSection) {
-  var rtcpParameters = {};
+  const rtcpParameters = {};
 
-  // Gets the first SSRC. Note tha with RTX there might be multiple
+  // Gets the first SSRC. Note that with RTX there might be multiple
   // SSRCs.
-  var remoteSsrc = SDPUtils.matchPrefix(mediaSection, 'a=ssrc:')
-    .map(function(line) {
-      return SDPUtils.parseSsrcMedia(line);
-    })
-    .filter(function(obj) {
-      return obj.attribute === 'cname';
-    })[0];
+  const remoteSsrc = SDPUtils.matchPrefix(mediaSection, 'a=ssrc:')
+    .map(line => SDPUtils.parseSsrcMedia(line))
+    .filter(obj => obj.attribute === 'cname')[0];
   if (remoteSsrc) {
     rtcpParameters.cname = remoteSsrc.value;
     rtcpParameters.ssrc = remoteSsrc.ssrc;
@@ -571,34 +573,46 @@ SDPUtils.parseRtcpParameters = function(mediaSection) {
 
   // Edge uses the compound attribute instead of reducedSize
   // compound is !reducedSize
-  var rsize = SDPUtils.matchPrefix(mediaSection, 'a=rtcp-rsize');
+  const rsize = SDPUtils.matchPrefix(mediaSection, 'a=rtcp-rsize');
   rtcpParameters.reducedSize = rsize.length > 0;
   rtcpParameters.compound = rsize.length === 0;
 
   // parses the rtcp-mux attrіbute.
   // Note that Edge does not support unmuxed RTCP.
-  var mux = SDPUtils.matchPrefix(mediaSection, 'a=rtcp-mux');
+  const mux = SDPUtils.matchPrefix(mediaSection, 'a=rtcp-mux');
   rtcpParameters.mux = mux.length > 0;
 
   return rtcpParameters;
 };
 
+SDPUtils.writeRtcpParameters = function(rtcpParameters) {
+  let sdp = '';
+  if (rtcpParameters.reducedSize) {
+    sdp += 'a=rtcp-rsize\r\n';
+  }
+  if (rtcpParameters.mux) {
+    sdp += 'a=rtcp-mux\r\n';
+  }
+  if (rtcpParameters.ssrc !== undefined && rtcpParameters.cname) {
+    sdp += 'a=ssrc:' + rtcpParameters.ssrc +
+      ' cname:' + rtcpParameters.cname + '\r\n';
+  }
+  return sdp;
+};
+
+
 // parses either a=msid: or a=ssrc:... msid lines and returns
 // the id of the MediaStream and MediaStreamTrack.
 SDPUtils.parseMsid = function(mediaSection) {
-  var parts;
-  var spec = SDPUtils.matchPrefix(mediaSection, 'a=msid:');
+  let parts;
+  const spec = SDPUtils.matchPrefix(mediaSection, 'a=msid:');
   if (spec.length === 1) {
     parts = spec[0].substr(7).split(' ');
     return {stream: parts[0], track: parts[1]};
   }
-  var planB = SDPUtils.matchPrefix(mediaSection, 'a=ssrc:')
-    .map(function(line) {
-      return SDPUtils.parseSsrcMedia(line);
-    })
-    .filter(function(msidParts) {
-      return msidParts.attribute === 'msid';
-    });
+  const planB = SDPUtils.matchPrefix(mediaSection, 'a=ssrc:')
+    .map(line => SDPUtils.parseSsrcMedia(line))
+    .filter(msidParts => msidParts.attribute === 'msid');
   if (planB.length > 0) {
     parts = planB[0].value.split(' ');
     return {stream: parts[0], track: parts[1]};
@@ -609,32 +623,32 @@ SDPUtils.parseMsid = function(mediaSection) {
 // parses draft-ietf-mmusic-sctp-sdp-26 first and falls back
 // to draft-ietf-mmusic-sctp-sdp-05
 SDPUtils.parseSctpDescription = function(mediaSection) {
-  var mline = SDPUtils.parseMLine(mediaSection);
-  var maxSizeLine = SDPUtils.matchPrefix(mediaSection, 'a=max-message-size:');
-  var maxMessageSize;
+  const mline = SDPUtils.parseMLine(mediaSection);
+  const maxSizeLine = SDPUtils.matchPrefix(mediaSection, 'a=max-message-size:');
+  let maxMessageSize;
   if (maxSizeLine.length > 0) {
     maxMessageSize = parseInt(maxSizeLine[0].substr(19), 10);
   }
   if (isNaN(maxMessageSize)) {
     maxMessageSize = 65536;
   }
-  var sctpPort = SDPUtils.matchPrefix(mediaSection, 'a=sctp-port:');
+  const sctpPort = SDPUtils.matchPrefix(mediaSection, 'a=sctp-port:');
   if (sctpPort.length > 0) {
     return {
       port: parseInt(sctpPort[0].substr(12), 10),
       protocol: mline.fmt,
-      maxMessageSize: maxMessageSize
+      maxMessageSize,
     };
   }
-  var sctpMapLines = SDPUtils.matchPrefix(mediaSection, 'a=sctpmap:');
+  const sctpMapLines = SDPUtils.matchPrefix(mediaSection, 'a=sctpmap:');
   if (sctpMapLines.length > 0) {
-    var parts = SDPUtils.matchPrefix(mediaSection, 'a=sctpmap:')[0]
+    const parts = sctpMapLines[0]
       .substr(10)
       .split(' ');
     return {
       port: parseInt(parts[0], 10),
       protocol: parts[1],
-      maxMessageSize: maxMessageSize
+      maxMessageSize,
     };
   }
 };
@@ -645,18 +659,18 @@ SDPUtils.parseSctpDescription = function(mediaSection) {
 // as the draft-ietf-mmusic-sctp-sdp-05 format (indicated by the m-line
 // protocol of DTLS/SCTP -- without UDP/ or TCP/)
 SDPUtils.writeSctpDescription = function(media, sctp) {
-  var output = [];
+  let output = [];
   if (media.protocol !== 'DTLS/SCTP') {
     output = [
       'm=' + media.kind + ' 9 ' + media.protocol + ' ' + sctp.protocol + '\r\n',
       'c=IN IP4 0.0.0.0\r\n',
-      'a=sctp-port:' + sctp.port + '\r\n'
+      'a=sctp-port:' + sctp.port + '\r\n',
     ];
   } else {
     output = [
       'm=' + media.kind + ' 9 ' + media.protocol + ' ' + sctp.port + '\r\n',
       'c=IN IP4 0.0.0.0\r\n',
-      'a=sctpmap:' + sctp.port + ' ' + sctp.protocol + ' 65535\r\n'
+      'a=sctpmap:' + sctp.port + ' ' + sctp.protocol + ' 65535\r\n',
     ];
   }
   if (sctp.maxMessageSize !== undefined) {
@@ -673,20 +687,20 @@ SDPUtils.generateSessionId = function() {
   return Math.random().toString().substr(2, 21);
 };
 
-// Write boilder plate for start of SDP
+// Write boiler plate for start of SDP
 // sessId argument is optional - if not supplied it will
 // be generated randomly
 // sessVersion is optional and defaults to 2
 // sessUser is optional and defaults to 'thisisadapterortc'
 SDPUtils.writeSessionBoilerplate = function(sessId, sessVer, sessUser) {
-  var sessionId;
-  var version = sessVer !== undefined ? sessVer : 2;
+  let sessionId;
+  const version = sessVer !== undefined ? sessVer : 2;
   if (sessId) {
     sessionId = sessId;
   } else {
     sessionId = SDPUtils.generateSessionId();
   }
-  var user = sessUser || 'thisisadapterortc';
+  const user = sessUser || 'thisisadapterortc';
   // FIXME: sess-id should be an NTP timestamp.
   return 'v=0\r\n' +
       'o=' + user + ' ' + sessionId + ' ' + version +
@@ -695,65 +709,11 @@ SDPUtils.writeSessionBoilerplate = function(sessId, sessVer, sessUser) {
       't=0 0\r\n';
 };
 
-SDPUtils.writeMediaSection = function(transceiver, caps, type, stream) {
-  var sdp = SDPUtils.writeRtpDescription(transceiver.kind, caps);
-
-  // Map ICE parameters (ufrag, pwd) to SDP.
-  sdp += SDPUtils.writeIceParameters(
-    transceiver.iceGatherer.getLocalParameters());
-
-  // Map DTLS parameters to SDP.
-  sdp += SDPUtils.writeDtlsParameters(
-    transceiver.dtlsTransport.getLocalParameters(),
-    type === 'offer' ? 'actpass' : 'active');
-
-  sdp += 'a=mid:' + transceiver.mid + '\r\n';
-
-  if (transceiver.direction) {
-    sdp += 'a=' + transceiver.direction + '\r\n';
-  } else if (transceiver.rtpSender && transceiver.rtpReceiver) {
-    sdp += 'a=sendrecv\r\n';
-  } else if (transceiver.rtpSender) {
-    sdp += 'a=sendonly\r\n';
-  } else if (transceiver.rtpReceiver) {
-    sdp += 'a=recvonly\r\n';
-  } else {
-    sdp += 'a=inactive\r\n';
-  }
-
-  if (transceiver.rtpSender) {
-    // spec.
-    var msid = 'msid:' + stream.id + ' ' +
-        transceiver.rtpSender.track.id + '\r\n';
-    sdp += 'a=' + msid;
-
-    // for Chrome.
-    sdp += 'a=ssrc:' + transceiver.sendEncodingParameters[0].ssrc +
-        ' ' + msid;
-    if (transceiver.sendEncodingParameters[0].rtx) {
-      sdp += 'a=ssrc:' + transceiver.sendEncodingParameters[0].rtx.ssrc +
-          ' ' + msid;
-      sdp += 'a=ssrc-group:FID ' +
-          transceiver.sendEncodingParameters[0].ssrc + ' ' +
-          transceiver.sendEncodingParameters[0].rtx.ssrc +
-          '\r\n';
-    }
-  }
-  // FIXME: this should be written by writeRtpDescription.
-  sdp += 'a=ssrc:' + transceiver.sendEncodingParameters[0].ssrc +
-      ' cname:' + SDPUtils.localCName + '\r\n';
-  if (transceiver.rtpSender && transceiver.sendEncodingParameters[0].rtx) {
-    sdp += 'a=ssrc:' + transceiver.sendEncodingParameters[0].rtx.ssrc +
-        ' cname:' + SDPUtils.localCName + '\r\n';
-  }
-  return sdp;
-};
-
 // Gets the direction from the mediaSection or the sessionpart.
 SDPUtils.getDirection = function(mediaSection, sessionpart) {
   // Look for sendrecv, sendonly, recvonly, inactive, default to sendrecv.
-  var lines = SDPUtils.splitLines(mediaSection);
-  for (var i = 0; i < lines.length; i++) {
+  const lines = SDPUtils.splitLines(mediaSection);
+  for (let i = 0; i < lines.length; i++) {
     switch (lines[i]) {
       case 'a=sendrecv':
       case 'a=sendonly':
@@ -771,8 +731,8 @@ SDPUtils.getDirection = function(mediaSection, sessionpart) {
 };
 
 SDPUtils.getKind = function(mediaSection) {
-  var lines = SDPUtils.splitLines(mediaSection);
-  var mline = lines[0].split(' ');
+  const lines = SDPUtils.splitLines(mediaSection);
+  const mline = lines[0].split(' ');
   return mline[0].substr(2);
 };
 
@@ -781,26 +741,26 @@ SDPUtils.isRejected = function(mediaSection) {
 };
 
 SDPUtils.parseMLine = function(mediaSection) {
-  var lines = SDPUtils.splitLines(mediaSection);
-  var parts = lines[0].substr(2).split(' ');
+  const lines = SDPUtils.splitLines(mediaSection);
+  const parts = lines[0].substr(2).split(' ');
   return {
     kind: parts[0],
     port: parseInt(parts[1], 10),
     protocol: parts[2],
-    fmt: parts.slice(3).join(' ')
+    fmt: parts.slice(3).join(' '),
   };
 };
 
 SDPUtils.parseOLine = function(mediaSection) {
-  var line = SDPUtils.matchPrefix(mediaSection, 'o=')[0];
-  var parts = line.substr(2).split(' ');
+  const line = SDPUtils.matchPrefix(mediaSection, 'o=')[0];
+  const parts = line.substr(2).split(' ');
   return {
     username: parts[0],
     sessionId: parts[1],
     sessionVersion: parseInt(parts[2], 10),
     netType: parts[3],
     addressType: parts[4],
-    address: parts[5]
+    address: parts[5],
   };
 };
 
@@ -809,8 +769,8 @@ SDPUtils.isValidSDP = function(blob) {
   if (typeof blob !== 'string' || blob.length === 0) {
     return false;
   }
-  var lines = SDPUtils.splitLines(blob);
-  for (var i = 0; i < lines.length; i++) {
+  const lines = SDPUtils.splitLines(blob);
+  for (let i = 0; i < lines.length; i++) {
     if (lines[i].length < 2 || lines[i].charAt(1) !== '=') {
       return false;
     }
