@@ -200,6 +200,12 @@ describe('fmtp', () => {
         parameters: {'0-15': undefined}});
       expect(out).to.equal('a=fmtp:100 0-15\r\n');
     });
+
+    it('serializes number 0 correctly', () => {
+      const out = SDPUtils.writeFmtp({payloadType: 100,
+        parameters: {'profile-id': 0}});
+      expect(out).to.equal('a=fmtp:100 profile-id=0\r\n');
+    });
   });
 });
 
@@ -471,19 +477,37 @@ describe('parseRtcpParameters', () => {
   });
 });
 
+describe('writeRtcpParameters', () => {
+  it('adds a rtcp-mux line if mux is true', () => {
+    const serialized = SDPUtils.writeRtcpParameters({mux: true});
+    expect(serialized).to.contain('a=rtcp-mux');
+  });
+
+  it('adds a rtcp-rsize line if reducedSize is true', () => {
+    const serialized = SDPUtils.writeRtcpParameters({reducedSize: true});
+    expect(serialized).to.contain('a=rtcp-rsize');
+  });
+
+  it('adds a a=ssrc line with cname if cname and ssrc are set', () => {
+    const serialized = SDPUtils.writeRtcpParameters({ssrc: 1, cname: 'foo'});
+    expect(serialized).to.contain('a=ssrc:1 cname:foo');
+  });
+});
+
+
 describe('parseFingerprint', () => {
   const res = SDPUtils.parseFingerprint('a=fingerprint:ALG fp');
   it('parses and lowercaseѕ the algorithm', () => {
     expect(res.algorithm).to.equal('alg');
   });
   it('parses the fingerprint value', () => {
-    expect(res.value).to.equal('fp');
+    expect(res.value).to.equal('FP');
   });
 });
 
 describe('getDtlsParameters', () => {
   const fp = 'a=fingerprint:sha-256 so:me:th:in:g1\r\n' +
-      'a=fingerprint:SHA-1 somethingelse';
+      'a=fingerprint:SHA-1 SOMETHINGELSE';
   const dtlsParameters = SDPUtils.getDtlsParameters(fp, '');
 
   it('sets the role to auto', () => {
@@ -500,8 +524,8 @@ describe('getDtlsParameters', () => {
   });
 
   it('extracts the fingerprints', () => {
-    expect(dtlsParameters.fingerprints[0].value).to.equal('so:me:th:in:g1');
-    expect(dtlsParameters.fingerprints[1].value).to.equal('somethingelse');
+    expect(dtlsParameters.fingerprints[0].value).to.equal('SO:ME:TH:IN:G1');
+    expect(dtlsParameters.fingerprints[1].value).to.equal('SOMETHINGELSE');
   });
 });
 
@@ -529,6 +553,12 @@ describe('getIceParameters', () => {
     expect(ice).to.be.an('Object');
   });
 
+  it('returns null when there are no ice-ufrag and ice-pwd', () => {
+    expect(SDPUtils.getIceParameters('a=ice-ufrag:123', '')).to.equal(null);
+    expect(SDPUtils.getIceParameters('a=ice-pwd:123', '')).to.equal(null);
+    expect(SDPUtils.getIceParameters('', '')).to.equal(null);
+  });
+
   it('parses the ufrag', () => {
     expect(ice.usernameFragment).to.equal('npaLWmWDg3Yp6vJt');
   });
@@ -538,10 +568,11 @@ describe('getIceParameters', () => {
 });
 
 describe('writeIceParameters', () => {
-  const serialized = SDPUtils.writeIceParameters({
+  const parameters = {
     usernameFragment: 'foo',
     password: 'bar'
-  });
+  };
+  const serialized = SDPUtils.writeIceParameters(parameters);
 
   it('serializes the usernameFragment', () => {
     expect(serialized).to.contain('a=ice-ufrag:foo');
@@ -550,6 +581,12 @@ describe('writeIceParameters', () => {
   it('serializes the password', () => {
     expect(serialized).to.contain('a=ice-pwd:bar');
   });
+
+  it('includes ice-lite when іceLite is true', () => {
+    const lite = SDPUtils.writeIceParameters(Object.assign({}, parameters,
+      {iceLite: true}));
+    expect(lite).to.contain('a=ice-lite');
+  });
 });
 
 describe('getMid', () => {
@@ -635,9 +672,11 @@ describe('ice candidate', () => {
     it('parses foundation', () => {
       expect(candidate.foundation).to.equal('702786350');
     });
+
     it('parses component', () => {
-      expect(candidate.component).to.equal(2);
+      expect(candidate.component).to.equal('rtcp');
     });
+
     it('parses priority', () => {
       expect(candidate.priority).to.equal(41819902, 'parses priority');
     });
@@ -686,10 +725,20 @@ describe('ice candidate', () => {
       expect(candidate.generation).to.equal('0');
     });
 
+    it('does not overwrite main attributes with extension attribues', () => {
+      expect(SDPUtils.parseCandidate(candidateString + ' type newtype').type)
+        .to.equal('relay');
+    });
+
     it('parses the candidate with the legacy a= prefix', () => {
       expect(SDPUtils.parseCandidate('a=' + candidateString).foundation)
         .to.equal('702786350');
     });
+
+    it('parses unknown components', () => {
+      expect(SDPUtils.parseCandidate(candidateString.replace(' 2 ', ' 3 '))
+        .component).to.equal('3');
+    });
   });
 
   describe('serialization', () => {
@@ -749,6 +798,19 @@ describe('ice candidate', () => {
       expect(serialized).to.equal('candidate:702786350 2 UDP 4189902 8.8.8.8 ' +
           '60769 typ host');
     });
+
+    it('translates component rtp to 1', () => {
+      candidate.component = 'rtp';
+      serialized = SDPUtils.writeCandidate(candidate).trim();
+      expect(serialized).to.equal('candidate:702786350 1 UDP 4189902 8.8.8.8 ' +
+          '60769 typ host');
+    });
+    it('translates component rtcp to 2', () => {
+      candidate.component = 'rtcp';
+      serialized = SDPUtils.writeCandidate(candidate).trim();
+      expect(serialized).to.equal('candidate:702786350 2 UDP 4189902 8.8.8.8 ' +
+          '60769 typ host');
+    });
   });
 });
 
@@ -1019,3 +1081,139 @@ describe('isValidSDP', () => {
   it('returns true for valid sdp', () =>
     expect(SDPUtils.isValidSDP(videoSDP)).to.equal(true));
 });
+
+describe('crypto', () => {
+  describe('parseCryptoKeyParams', () => {
+    const result = SDPUtils.parseCryptoKeyParams(
+      'inline:d0RmdmcmVCspeEc3QGZiNWpVLFJhQX1cfHAwJSoj|2^20|1:4');
+    it('returns null if the key method is not `inline`', () => {
+      expect(SDPUtils.parseCryptoKeyParams('foo')).to.equal(null);
+    });
+    it('keyMethod', () => {
+      expect(result.keyMethod).to.equal('inline');
+    });
+    it('keySalt', () => {
+      expect(result.keySalt).to.equal(
+        'd0RmdmcmVCspeEc3QGZiNWpVLFJhQX1cfHAwJSoj');
+    });
+    it('lifeTime', () => {
+      expect(result.lifeTime).to.equal('2^20');
+    });
+    it('mkiValue', () => {
+      expect(result.mkiValue).to.equal('1');
+    });
+    it('mkiLength', () => {
+      expect(result.mkiLength).to.equal('4');
+    });
+  });
+
+  describe('writeCryptoKeyParams', () => {
+    it('serializes keyMethod and keySalt', () => {
+      expect(SDPUtils.writeCryptoKeyParams({
+        keyMethod: 'inline',
+        keySalt: 'ks',
+      })).to.equal('inline:ks');
+    });
+    it('serializes lifeTime if present', () => {
+      expect(SDPUtils.writeCryptoKeyParams({
+        keyMethod: 'inline',
+        keySalt: 'ks',
+        lifeTime: '2^24',
+      })).to.equal('inline:ks|2^24');
+    });
+    it('serializes mkiValue and mkiLength if present', () => {
+      expect(SDPUtils.writeCryptoKeyParams({
+        keyMethod: 'inline',
+        keySalt: 'ks',
+        lifeTime: '2^24',
+        mkiValue: '1',
+        mkiLength: '4',
+      })).to.equal('inline:ks|2^24|1:4');
+    });
+    it('serializes mkiValue and mkiLength if lifeTime is not present', () => {
+      expect(SDPUtils.writeCryptoKeyParams({
+        keyMethod: 'inline',
+        keySalt: 'ks',
+        mkiValue: '1',
+        mkiLength: '4',
+      })).to.equal('inline:ks|1:4');
+    });
+  });
+
+  describe('parseCryptoLine', () => {
+    const result = SDPUtils.parseCryptoLine('a=crypto:0 ' +
+      'AES_CM_128_HMAC_SHA1_80 ' +
+      'inline:gd0dcDM6tVrk2sqLuNYp9EFX0WYMNu7kQw/V0s23');
+    it('parses the tag', () => {
+      expect(result.tag).to.equal(0);
+    });
+    it('parses the cryptoSuite', () => {
+      expect(result.cryptoSuite).to.equal('AES_CM_128_HMAC_SHA1_80');
+    });
+    it('parses the keyParams as string', () => {
+      expect(result.keyParams).to.equal(
+        'inline:gd0dcDM6tVrk2sqLuNYp9EFX0WYMNu7kQw/V0s23');
+    });
+    describe('parses sessionParams', () => {
+      it('as empty array when there are no sessionParams', () => {
+        expect(result.sessionParams).to.be.an('array');
+        expect(result.sessionParams).to.have.length(0);
+      });
+      it('as array', () => {
+        expect(SDPUtils.parseCryptoLine('a=crypto:0 ' +
+          'cryptoSuite keyParams foo bar').sessionParams)
+          .to.deep.equal(['foo', 'bar']);
+      });
+    });
+  });
+
+  describe('writeCryptoLine', () => {
+    it('generates the expected output', () => {
+      const out = SDPUtils.writeCryptoLine({
+        tag: 1,
+        cryptoSuite: 'something',
+        keyParams: 'params',
+        sessionParams: undefined,
+      });
+      expect(out).to.equal('a=crypto:1 something params\r\n');
+    });
+
+    it('serializes keyParams', () => {
+      const out = SDPUtils.writeCryptoLine({
+        tag: 1,
+        cryptoSuite: 'something',
+        keyParams: {
+          keyMethod: 'inline',
+          keySalt: 'ks',
+        },
+      });
+      expect(out).to.equal('a=crypto:1 something inline:ks\r\n');
+    });
+
+    it('serializes sessionParams', () => {
+      const out = SDPUtils.writeCryptoLine({
+        tag: 1,
+        cryptoSuite: 'something',
+        keyParams: 'params',
+        sessionParams: ['foo', 'bar'],
+      });
+      expect(out).to.equal('a=crypto:1 something params foo bar\r\n');
+    });
+  });
+
+  describe('getCryptoParameters', () => {
+    const result = SDPUtils.getCryptoParameters(
+      'a=crypto:0 suite1 params1\r\n' +
+      'a=crypto:1 suite2 params2\r\n'
+    );
+    it('returns an array', () => {
+      expect(result).to.be.an('Array');
+    });
+    it('matches the number of input lines', () => {
+      expect(result).to.have.length(2);
+    });
+    it('returns two SDESParameters entries with numeric tags', () => {
+      expect(result.map(r => r.tag)).to.deep.equal([0, 1]);
+    });
+  });
+});
